#include <iostream>
#include <string>
#include <list>
using namespace std;

// -----------------------   Persona  -----------------------------------
class Persona {
private:
	string apellido1;
	string apellido2;
	string nombre;
	int DNI;
	string estado_civil;
protected:
	Persona(string a1="", string a2="", string n="",
	    int id=0, string e=""):
		apellido1(a1), apellido2(a2), nombre(n), DNI(id),
		estado_civil(e) {}
public:
	void cambiar_estado_civil(string e);
	string obtener_estado_civil() const;
	void imprimir() const;
	int obtener_id() const;
	string nombre_apellidos() const;
};

string Persona::nombre_apellidos() const {
	return (nombre + " " + apellido1 + " " + apellido2);
}
int Persona::obtener_id() const {
	return DNI;
}

void Persona::cambiar_estado_civil(string e) {
	estado_civil = e;
}
string Persona::obtener_estado_civil() const {
	return estado_civil;
}

void Persona::imprimir() const{
	cout << "Datos personales" << endl;
	cout << "\tNombre y apellidos: " << nombre << " " <<
		apellido1 << " " << apellido2 << " " << endl;
	cout << "\tDNI: " << DNI << endl;
	cout << "\tEstado civil: " << estado_civil << endl;
}

// -----------------------   Estudiante  -----------------------------------
class Estudiante: public Persona {
private:
	int curso;
public:
	Estudiante(string a1="", string a2="", string n="",
	    int id=0, string e="", int c=0):
		Persona(a1, a2, n, id, e), curso(c) {}
	void imprimir() const;
	void cambiar_curso(int c);
};

void Estudiante::imprimir() const{
	Persona::imprimir();
	cout << "Datos academicos" << endl;
	cout << "\tCurso: " << curso << endl;
}

void Estudiante::cambiar_curso(int c) {
	curso=c;
}

// -----------------------   Empleado  -----------------------------------
class Empleado: public Persona {
private:
	int fecha_alta;
	int despacho;
protected:
	Empleado(string a1="", string a2="", string n="",
	    int id=0, string e="", int fa=1900, int d=0):
		Persona(a1, a2, n, id, e), fecha_alta(fa), despacho(d) {}
public:
	int obtener_fecha_alta() const;
	void imprimir() const;
	void cambiar_despacho(int d);
};

int Empleado::obtener_fecha_alta() const {
	return fecha_alta;
}
void Empleado::imprimir() const{
	Persona::imprimir();
	cout << "Datos del trabajador" << endl;
	cout << "\tFecha de alta: " << fecha_alta << endl;
	cout << "\tDespacho: " << despacho << endl;
}
void Empleado::cambiar_despacho(int d) {
	despacho=d;
}

// -----------------------   Profesor  -----------------------------------
class Profesor: public Empleado {
private:
	string dpto;
	static int const sueldo_base=1000;
public:
	Profesor(string a1="", string a2="", string n="",
	    int id=0, string e="", int fa=1900, int d=0, string dep=""):
		Empleado(a1, a2, n, id, e, fa, d), dpto(dep) {}
	void imprimir() const;
	void cambiar_dpto(string dep);
	int salario();
};

void Profesor::imprimir() const{
	Empleado::imprimir();
	cout << "Datos laborales" << endl;
	cout << "\tDepartamento: " << dpto << endl;
}

void Profesor::cambiar_dpto(string dep) {
	dpto=dep;

}
int Profesor::salario() {
	return  ((obtener_fecha_alta()>=2000)?sueldo_base:(sueldo_base*108/100));
}

// -----------------------   PAS  -----------------------------------
class PAS: public Empleado {
private:
	string seccion;
	static int const sueldo_base=1000;
public:
	PAS(string a1="", string a2="", string n="",
	    int id=0, string e="", int fa=1900, int d=0, string s=""):
		Empleado(a1, a2, n, id, e, fa, d), seccion(s) {}
	void imprimir() const;
	void cambiar_seccion(string sec);
	int salario();
};

void PAS::imprimir() const{
	Empleado::imprimir();
	cout << "Datos laborales" << endl;
	cout << "\tSección: " << seccion << endl;
}

void PAS::cambiar_seccion(string sec) {
	seccion=sec;

}

int PAS::salario() {
	return  ((obtener_estado_civil()!="casado")?sueldo_base:(sueldo_base*105/100));
}

// -----------------------   Centro  -----------------------------------
class Centro {
private:
	list<Estudiante *> estudiantes;
	list<Profesor *> personal_docente;
	list<PAS *> personal_servicio;
public:
	void alta(const Estudiante &e);
	void alta(const Profesor &p);
	void alta(const PAS &s);
	bool baja(int DNI);
	void imprimir_personas();
	void imprimir_nomina();
};

void Centro::alta(const Estudiante &e){
	estudiantes.push_back(new Estudiante(e));
}
void Centro::alta(const Profesor &p){
	personal_docente.push_back(new Profesor(p));
}
void Centro::alta(const PAS &s){
	personal_servicio.push_back(new PAS(s));
}

template <typename T> bool baja_lista(list<T *> &lista, int id) {
	for (typename list<T *>::iterator i=lista.begin();
		i!=lista.end(); i++)
		if ((*i)->obtener_id()==id) {
			delete(*i);
			lista.erase(i);
			return true;
		}
	return false;
}
template <typename T> void imprimir_lista(list<T *> &lista) {
	for (typename list<T *>::const_iterator i=lista.begin();
		i!=lista.end(); i++)
		(*i)->imprimir();
}
void Centro::imprimir_personas(){
	imprimir_lista(estudiantes);
	imprimir_lista(personal_docente);
	imprimir_lista(personal_servicio);
} 
bool Centro::baja(int id){
	return baja_lista(estudiantes, id) || baja_lista(personal_docente, id)
		|| baja_lista(personal_servicio, id);
}
template <typename T> void nomina(list<T *> &lista) {
	for (typename list<T *>::const_iterator i=lista.begin();
		i!=lista.end(); i++) {
		cout << "Nombre y apellidos: " << (*i)->nombre_apellidos()
		<< " Nómina: " << (*i)->salario() << endl;
	}
}
void Centro::imprimir_nomina(){
	nomina(personal_docente);
	nomina(personal_servicio);
} 

// -----------------------   main  -----------------------------------

int main () {
	// Persona p; // error de compilacion
	//Empleado e; // error de compilacion
	Estudiante e1("Martinez", "Gomez", "Juan", 12345678, "soltero", 3);
	Profesor p1("Garcia", "Perez", "Luisa", 87654321, "divorciado",
		1999, 511, "Matematicas");
	PAS s1("Fernandez", "Lopez", "Maria", 8888888, "casado",
		1990, 111, "Biblioteca");
	Centro facultad;

	facultad.alta(e1);
	facultad.alta(p1);
	facultad.alta(s1);
	facultad.imprimir_personas();
	facultad.imprimir_nomina();

	return 0;
}
