#include <iostream>
#include <string>
#include <list>
using namespace std;

// -----------------------   Persona  -----------------------------------
class Persona {
private:
	string apellido1;
	string apellido2;
	string nombre;
	int DNI;
	string estado_civil;
protected:
	Persona(string a1="", string a2="", string n="",
	    int id=0, string e=""):
		apellido1(a1), apellido2(a2), nombre(n), DNI(id),
		estado_civil(e) {}
public:
	void cambiar_estado_civil(string e);
	string obtener_estado_civil() const;
	virtual void imprimir() const;
	int obtener_id() const;
	virtual Persona * clona_persona ()=0;
	virtual ~Persona() {}
	string nombre_apellidos() const;
};

string Persona::obtener_estado_civil() const {
        return estado_civil;
}

string Persona::nombre_apellidos() const {
        return (nombre + " " + apellido1 + " " + apellido2);
}

int Persona::obtener_id() const {
	return DNI;
}

void Persona::cambiar_estado_civil(string e) {
	estado_civil = e;
}

void Persona::imprimir() const{
	cout << "Datos personales" << endl;
	cout << "\tNombre y apellidos: " << nombre << " " <<
		apellido1 << " " << apellido2 << " " << endl;
	cout << "\tDNI: " << DNI << endl;
	cout << "\tEstado civil: " << estado_civil << endl;
}

// -----------------------   Estudiante  -----------------------------------
class Estudiante: public Persona {
private:
	int curso;
public:
	Estudiante(string a1="", string a2="", string n="",
	    int id=0, string e="", int c=0):
		Persona(a1, a2, n, id, e), curso(c) {}
	void imprimir() const;
	void cambiar_curso(int c);
	Persona *clona_persona ();
};

Persona *Estudiante::clona_persona () {
	return new Estudiante(*this);
}

void Estudiante::imprimir() const{
	Persona::imprimir();
	cout << "Datos academicos" << endl;
	cout << "\tCurso: " << curso << endl;
}

void Estudiante::cambiar_curso(int c) {
	curso=c;
}

// -----------------------   Empleado  -----------------------------------
class Empleado: public Persona {
private:
	int fecha_alta;
	int despacho;
protected:
	Empleado(string a1="", string a2="", string n="",
	    int id=0, string e="", int fa=1900, int d=0):
		Persona(a1, a2, n, id, e), fecha_alta(fa), despacho(d) {}
public:
	void imprimir() const;
	void cambiar_despacho(int d);
	int obtener_fecha_alta() const;
	virtual int salario()=0;
	virtual ~Empleado() {}
};

void Empleado::imprimir() const{
	Persona::imprimir();
	cout << "Datos del trabajador" << endl;
	cout << "\tFecha de alta: " << fecha_alta << endl;
	cout << "\tDespacho: " << despacho << endl;
}
void Empleado::cambiar_despacho(int d) {
	despacho=d;
}
int Empleado::obtener_fecha_alta() const {
	return fecha_alta;
}

// -----------------------   Profesor  -----------------------------------
class Profesor: public Empleado {
private:
	string dpto;
	static int const sueldo_base=1000;
public:
	Profesor(string a1="", string a2="", string n="",
	    int id=0, string e="", int fa=1900, int d=0, string dep=""):
		Empleado(a1, a2, n, id, e, fa, d), dpto(dep) {}
	void imprimir() const;
	void cambiar_dpto(string dep);
	Persona *clona_persona ();
	int salario();
};

Persona *Profesor::clona_persona () {
	return new Profesor(*this);
}

void Profesor::imprimir() const{
	Empleado::imprimir();
	cout << "Datos laborales" << endl;
	cout << "\tDepartamento: " << dpto << endl;
}

void Profesor::cambiar_dpto(string dep) {
	dpto=dep;

}

int Profesor::salario() {
        return  ((obtener_fecha_alta()>=2000)?sueldo_base:(sueldo_base*108/100));
}


// -----------------------   PAS  -----------------------------------
class PAS: public Empleado {
private:
	string seccion;
	static int const sueldo_base=1000;
public:
	PAS(string a1="", string a2="", string n="",
	    int id=0, string e="", int fa=1900, int d=0, string s=""):
		Empleado(a1, a2, n, id, e, fa, d), seccion(s) {}
	void imprimir() const;
	void cambiar_seccion(string sec);
	Persona *clona_persona ();
	int salario();
};

Persona *PAS::clona_persona () {
	return new PAS(*this);
}
void PAS::imprimir() const{
	Empleado::imprimir();
	cout << "Datos laborales" << endl;
	cout << "\tSección: " << seccion << endl;
}

void PAS::cambiar_seccion(string sec) {
	seccion=sec;

}
int PAS::salario() {
        return  ((obtener_estado_civil()!="casado")?sueldo_base:(sueldo_base*105/100));
}


// -----------------------   Centro  -----------------------------------
class Centro {
private:
	list<Persona *> personas;
	list<Empleado *> empleados;
	
public:
	void alta(Persona &p);
	bool baja(int DNI);
	void imprimir_personas();
	void imprimir_nomina();
};

void Centro::alta(Persona &p){
	Persona *pp=p.clona_persona();
	personas.push_back(pp);
	Empleado *pe=dynamic_cast<Empleado *>(pp);
	if (pe)
		empleados.push_back(pe);
}

void Centro::imprimir_personas(){
	for (list<Persona *>::const_iterator i=personas.begin();
		i!=personas.end(); i++)
		(*i)->imprimir();
} 
bool Centro::baja(int id){
	for (list<Empleado *>::iterator i=empleados.begin();
		i!=empleados.end(); i++)
		if ((*i)->obtener_id()==id) {
			empleados.erase(i);
			break;
		}
	for (list<Persona *>::iterator i=personas.begin();
		i!=personas.end(); i++)
		if ((*i)->obtener_id()==id) {
			delete(*i);
			personas.erase(i);
			return true;
		}
	return false;
}
void Centro::imprimir_nomina(){
	for (list<Empleado *>::const_iterator i=empleados.begin();
                i!=empleados.end(); i++) {
                cout << "Nombre y apellidos: " << (*i)->nombre_apellidos()
                << " Nómina: " << (*i)->salario() << endl;
        }
}


// -----------------------   main  -----------------------------------

int main () {
	Estudiante e1("Martinez", "Gomez", "Juan", 12345678, "soltero", 3);
	Profesor p1("Garcia", "Perez", "Luisa", 87654321, "divorciado",
		1999, 511, "Matematicas");
	PAS s1("Fernandez", "Lopez", "Maria", 8888888, "casado",
		1990, 111, "Biblioteca");
	Centro facultad;

	facultad.alta(e1);
	facultad.alta(p1);
	facultad.alta(s1);
	facultad.imprimir_personas();
	facultad.baja(8888888);
	facultad.imprimir_personas();
	facultad.imprimir_nomina();

	return 0;
}
