#include <iostream>
#include <string>
#include <list>
using namespace std;

// -----------------------   Persona  -----------------------------------
class Persona {
private:
	string apellido1;
	string apellido2;
	string nombre;
	int DNI;
	string estado_civil;
protected:
	Persona(string a1="", string a2="", string n="",
	    int id=0, string e=""):
		apellido1(a1), apellido2(a2), nombre(n), DNI(id),
		estado_civil(e) {}
public:
	void cambiar_estado_civil(string e);
	string obtener_estado_civil() const;
	virtual void imprimir(ostream &os) const;
	virtual void leer(istream &is);
	int obtener_id() const;
	virtual Persona * clona_persona ()=0;
	string nombre_apellidos() const;
	virtual ~Persona() {}
};

string Persona::obtener_estado_civil() const {
        return estado_civil;
}

string Persona::nombre_apellidos() const {
        return (nombre + " " + apellido1 + " " + apellido2);
}

int Persona::obtener_id() const {
	return DNI;
}

void Persona::cambiar_estado_civil(string e) {
	estado_civil = e;
}

void Persona::imprimir(ostream &os) const{
	os << "Datos personales" << endl;
	os << "\tNombre y apellidos: " << nombre << " " <<
		apellido1 << " " << apellido2 << " " << endl;
	os << "\tDNI: " << DNI << endl;
	os << "\tEstado civil: " << estado_civil << endl;
}

void Persona::leer(istream &is) {
        cout << "Nombre: ";
        is >> nombre;
        cout << "Primer apellido: ";
        is >> apellido1;
        cout << "Segundo apellido: ";
        is >> apellido2;
        cout << "DNI: ";
        is >> DNI;
        cout << "Estado civil: ";
        is >> estado_civil;
}
ostream & operator<< (ostream &os, const Persona &p) {
        p.imprimir(os);
        return os;
}
istream & operator>> (istream &is, Persona &p) {
        p.leer(is);
        return is;
}

// -----------------------   Estudiante  -----------------------------------
class Estudiante: public Persona {
private:
	int curso;
public:
	Estudiante(string a1="", string a2="", string n="",
	    int id=0, string e="", int c=0):
		Persona(a1, a2, n, id, e), curso(c) {}
	void imprimir(ostream &os) const;
	void cambiar_curso(int c);
	Persona *clona_persona ();
	void leer(istream &is);
};

Persona *Estudiante::clona_persona () {
	return new Estudiante(*this);
}

void Estudiante::imprimir(ostream &os) const{
	Persona::imprimir(os);
	os << "Datos academicos" << endl;
	os << "\tCurso: " << curso << endl;
}

void Estudiante::cambiar_curso(int c) {
	curso=c;
}
void Estudiante::leer(istream &is) {
        Persona::leer(is);
        cout << "Curso: ";
        is >> curso;
}
ostream & operator<< (ostream &os, const Estudiante &e) {
        e.imprimir(os);
        return os;
}
istream & operator>> (istream &is, Estudiante &e) {
        e.leer(is);
        return is;
}


// -----------------------   Empleado  -----------------------------------
class Empleado: public Persona {
private:
	int fecha_alta;
	int despacho;
protected:
	Empleado(string a1="", string a2="", string n="",
	    int id=0, string e="", int fa=1900, int d=0):
		Persona(a1, a2, n, id, e), fecha_alta(fa), despacho(d) {}
public:
	void imprimir(ostream &os) const;
	void cambiar_despacho(int d);
	int obtener_fecha_alta() const;
	void leer(istream &is);
	virtual int salario()=0;
	virtual ~Empleado() {}
};

void Empleado::imprimir(ostream &os) const{
	Persona::imprimir(os);
	os << "Datos del trabajador" << endl;
	os << "\tFecha de alta: " << fecha_alta << endl;
	os << "\tDespacho: " << despacho << endl;
}
void Empleado::cambiar_despacho(int d) {
	despacho=d;
}
int Empleado::obtener_fecha_alta() const {
	return fecha_alta;
}

void Empleado::leer(istream &is) {
        Persona::leer(is);
        cout << "Fecha de alta: ";
        is >> fecha_alta;
        cout << "Despacho: ";
        is >> despacho;
}
ostream & operator<< (ostream &os, const Empleado &e) {
        e.imprimir(os);
        return os;
}
istream & operator>> (istream &is, Empleado &e) {
        e.leer(is);
        return is;
}

// -----------------------   Profesor  -----------------------------------
class Profesor: public Empleado {
private:
	string dpto;
	static int const sueldo_base=1000;
public:
	Profesor(string a1="", string a2="", string n="",
	    int id=0, string e="", int fa=1900, int d=0, string dep=""):
		Empleado(a1, a2, n, id, e, fa, d), dpto(dep) {}
	void imprimir(ostream &os) const;
	void cambiar_dpto(string dep);
	Persona *clona_persona ();
	int salario();
	void leer(istream &is);
};

Persona *Profesor::clona_persona () {
	return new Profesor(*this);
}

void Profesor::imprimir(ostream &os) const{
	Empleado::imprimir(os);
	os << "Datos laborales" << endl;
	os << "\tDepartamento: " << dpto << endl;
}

void Profesor::cambiar_dpto(string dep) {
	dpto=dep;

}

int Profesor::salario() {
        return  ((obtener_fecha_alta()>=2000)?sueldo_base:(sueldo_base*108/100));
}

void Profesor::leer(istream &is) {
        Empleado::leer(is);
        cout << "Departamento: ";
        is >> dpto;
}
ostream & operator<< (ostream &os, const Profesor &p) {
        p.imprimir(os);
        return os;
}
istream & operator>> (istream &is, Profesor &p) {
        p.leer(is);
        return is;
}


// -----------------------   PAS  -----------------------------------
class PAS: public Empleado {
private:
	string seccion;
	static int const sueldo_base=1000;
public:
	PAS(string a1="", string a2="", string n="",
	    int id=0, string e="", int fa=1900, int d=0, string s=""):
		Empleado(a1, a2, n, id, e, fa, d), seccion(s) {}
	void imprimir(ostream &os) const;
	void cambiar_seccion(string sec);
	Persona *clona_persona ();
	int salario();
	void leer(istream &is);
};

Persona *PAS::clona_persona () {
	return new PAS(*this);
}
void PAS::imprimir(ostream &os) const{
	Empleado::imprimir(os);
	os << "Datos laborales" << endl;
	os << "\tSección: " << seccion << endl;
}

void PAS::cambiar_seccion(string sec) {
	seccion=sec;

}
int PAS::salario() {
        return  ((obtener_estado_civil()!="casado")?sueldo_base:(sueldo_base*105/100));
}

void PAS::leer(istream &is) {
        Empleado::leer(is);
        cout << "Sección: ";
        is >> seccion;
}
ostream & operator<< (ostream &os, const PAS &p) {
        p.imprimir(os);
        return os;
}
istream & operator>> (istream &is, PAS &p) {
        p.leer(is);
        return is;
}


// -----------------------   Centro  -----------------------------------
class Centro {
private:
	list<Persona *> personas;
	list<Empleado *> empleados;
	
public:
	void alta(Persona &p);
	bool baja(int DNI);
	void imprimir_personas(ostream &os) const;
	void imprimir_nomina(ostream &os) const;
};

void Centro::alta(Persona &p){
	Persona *pp=p.clona_persona();
	personas.push_back(pp);
	Empleado *pe=dynamic_cast<Empleado *>(pp);
	if (pe)
		empleados.push_back(pe);
}

void Centro::imprimir_personas(ostream &os) const{
	for (list<Persona *>::const_iterator i=personas.begin();
		i!=personas.end(); i++)
		os<< *(*i);
} 
bool Centro::baja(int id){
	for (list<Empleado *>::iterator i=empleados.begin();
		i!=empleados.end(); i++)
		if ((*i)->obtener_id()==id) {
			empleados.erase(i);
			break;
		}
	for (list<Persona *>::iterator i=personas.begin();
		i!=personas.end(); i++)
		if ((*i)->obtener_id()==id) {
			delete(*i);
			personas.erase(i);
			return true;
		}
	return false;
}
void Centro::imprimir_nomina(ostream &os) const{
	for (list<Empleado *>::const_iterator i=empleados.begin();
                i!=empleados.end(); i++) {
                os << "Nombre y apellidos: " << (*i)->nombre_apellidos()
                << " Nómina: " << (*i)->salario() << endl;
        }
}

ostream & operator<< (ostream &os, const Centro &c) {
        c.imprimir_personas(os);
        return os;
}

// -----------------------   main  -----------------------------------

int main () {
	Estudiante e1;
	Profesor p1;
	PAS s1;
	Centro facultad;

	cin >> e1;
	facultad.alta(e1);
	cin >> p1;
	facultad.alta(p1);
	cin >> s1;
	facultad.alta(s1);
	cout << facultad;
	facultad.imprimir_nomina(cout);

	return 0;
}
